﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace reimers_tutorial06_series1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        GraphicsDevice device;

        Effect effect;

        VertexPositionColor[] vertices;

        VertexBuffer vb; // vertex buffer
        IndexBuffer ib; // index buffer

        float angle = 0.0f; // 전역 변수 정의

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();

            SetupXNADevice();

            SetupVertices();
            SetupIndices(); // 추가

            SetupCamera();
        }

        private void SetupXNADevice()
        {
            device = graphics.GraphicsDevice;

            graphics.PreferredBackBufferWidth = 500;
            graphics.PreferredBackBufferHeight = 500;

            graphics.IsFullScreen = false;

            graphics.ApplyChanges();

            Window.Title = "riemer`s XNA tutorial 06 - series 1";

            CompiledEffect compiledEffect = Effect.CompileEffectFromFile("@/../../../../effects.fx", null, null, CompilerOptions.None, TargetPlatform.Windows);
            effect = new Effect(graphics.GraphicsDevice, compiledEffect.GetEffectCode(), CompilerOptions.None, null);
        }

        private void SetupCamera()
        {
            Matrix viewMatrix = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 40.0f), new Vector3(0.0f, 0.0f, 0.0f), new Vector3(0.0f, 1.0f, 0.0f));
            Matrix projectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, Window.ClientBounds.Width / Window.ClientBounds.Height, 1.0f, 50.0f);

            effect.Parameters["xView"].SetValue(viewMatrix);
            effect.Parameters["xProjection"].SetValue(projectionMatrix);
            effect.Parameters["xWorld"].SetValue(Matrix.Identity);
        }

        private void SetupVertices()
        {
            vertices = new VertexPositionColor[5]; // vertex 개수 수정

            vertices[0].Position = new Vector3(0.0f, 0.0f, 0.0f); // vertex 좌표값 수정
            vertices[0].Color = Color.White; // vertex 색상값 수정
            vertices[1].Position = new Vector3(5.0f, 0.0f, 0.0f);
            vertices[1].Color = Color.White;
            vertices[2].Position = new Vector3(10.0f, 0.0f, 0.0f);
            vertices[2].Color = Color.White;
            vertices[3].Position = new Vector3(5.0f, 5.0f, 0.0f);
            vertices[3].Color = Color.White;
            vertices[4].Position = new Vector3(10.0f, 5.0f, 0.0f);
            vertices[4].Color = Color.White;

            vb = new VertexBuffer(device, VertexPositionColor.SizeInBytes * 5, ResourceUsage.WriteOnly, ResourceManagementMode.Automatic); // vertex buffer 생성
            vb.SetData(vertices); // vertex buffer에 배열을 저장
        }

        private void SetupIndices()
        {
            short[] indices = new short[6]; // index 정보를 저장할 배열 생성

            indices[0] = 3; // 채워넣는다.
            indices[1] = 1;
            indices[2] = 0;
            indices[3] = 4;
            indices[4] = 2;
            indices[5] = 1;

            ib = new IndexBuffer(device, typeof(short), 6, ResourceUsage.WriteOnly, ResourceManagementMode.Automatic); // index buffer 생성
            ib.SetData(indices); // index buffer에 배열을 저장
        }

        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            angle += 0.05f;

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            device.RenderState.FillMode = FillMode.WireFrame; // wireframe으로 보이게 한다.

            device.RenderState.CullMode = CullMode.None;

            device.Clear(Color.DarkSlateBlue);

            // TODO: Add your drawing code here

            effect.CurrentTechnique = effect.Techniques["Colored"];

            Matrix worldMatrix = Matrix.Identity; // 월드 좌표계 생성

            effect.Parameters["xWorld"].SetValue(worldMatrix);

            effect.Begin();

            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                device.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes); // vb를 디바이스의 vertex buffer로 설정
                device.Indices = ib; // ib를 디바이스의 index buffer로 설정
                device.VertexDeclaration = new VertexDeclaration(device, VertexPositionColor.VertexElements); // vertex의 속성을 설정
                device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, 5, 0, 2); // 삼각형을 그린다.
                
                pass.End();
            }

            effect.End();

            base.Draw(gameTime);
        }
    }
}
